<?php
/**
 * $Horde: troll/groupview.php,v 1.52 2005/01/03 14:35:35 jan Exp $
 *
 * Copyright 2000-2005 Charles J. Hagenbuch <chuck@horde.org>
 * Copyright 2000-2005 Jon Parise <jon@horde.org>
 * Copyright 2000-2005 Anil Madhavapeddy <avsm@horde.org>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

/* See if this is a header array or an array of header arrays. */
function _printThreadOverview($headers)
{
    if (!is_array($headers)) {
        return;
    }

    foreach ($headers as $hdr) {
        if (isset($hdr['author'])) {
            require TROLL_TEMPLATES . '/threadoverview/article-index.inc';
        } else {
            _printThreadOverview($hdr);
        }
    }
}

@define('TROLL_BASE', dirname(__FILE__));
require_once TROLL_BASE . '/lib/base.php';
require_once TROLL_BASE . '/lib/NNTP.php';

if (isset($language) && !empty($language)) {
    setlocale(LC_TIME, $language);
}

$art_anchor = array();
$curresults = 0;

$catchup = Util::getFormData('catchup', 'n');
$max = $maxresults = $prefs->getValue('max_articles');

/* Get the current group. */
$group = Util::getFormData('group', $prefs->getValue('last_group'));
$prefs->setValue('last_group', $group);

/* Make sure we have an NNTP stream. */
$troll_nntp = &Troll_NNTP::singleton();
$troll_nntp->changeGroup($group);
$articles = @imap_search($_SESSION['troll']['stream'], 'ALL', SE_UID);

if (!$prefs->getValue('show_read')) {
    /* Remove read messages from article list. */
    $articles = RemoveReadArticles($articles, $group);
}

if (!$articles) {
    Horde::logMessage("Error on imap_search: " . imap_last_error(), __FILE__, __LINE__, PEAR_LOG_NOTICE);
    $articles = array();
} else {
    sort($articles);
}

$article_count = count($articles);
if ($article_count > $maxresults) {
    $ind_min = $article_count - $maxresults;
    MarkReadUntil($articles[$ind_min - 1]);
    $articles = array_slice($articles, $ind_min);
}

$last_read = ImapFirst();

$max *= $prefs->getValue('expand_tree') ? 2 : 6;
$last = (Util::getFormData('end') > 0) ? Util::getFormData('end') : ImapLast();
if (Util::getFormData('only_new')) {
    $first = $last_read + 1;
} else {
    $first = ($last > $max) ? ($last - $max) : 0;
}
$end = $last;
if ($first < $last_read) {
    $first = $last_read;
}

if ($catchup == 'y') {
    if ($conf['news']['subscribe']) {
        MarkReadUntil($last);
    }
    $last += 1;
    $first = $last;
}

$firstprinted = $last;
$lastprinted = $first;

if ($articles) {
    $overview = @imap_fetch_overview($_SESSION['troll']['stream'], implode(',', $articles), FT_UID);
    if (!$overview) {
        Horde::fatal(PEAR::raiseError("Error on imap_fetch_overview: " . imap_last_error()), __FILE__, __LINE__, true);
    }
}

$title = _("Newsgroup List");

$newstart = ($maxresults > $end) ? 0 : $end - $maxresults;

$prevend = $firstprinted - $maxresults;
$nextend = $firstprinted + $maxresults;

$preva = $maxresults;
$nexta = $maxresults;

if ($nextend > ImapLast()) {
    $nextend = $last;
    $nexta = $last - $end;
}

if ($prevend < ImapFirst()) {
    $preva = 0;
}

$t_last  = ImapLast();
$t_first = ImapFirst();
$t_new   = $last_read;
if ($t_new < $t_first) {
    $t_new = $t_first;
}

$a_old = $t_new  - $t_first;
$a_new = $t_last - $t_new;

$refresh_url = Util::addParameter(Horde::selfUrl(), 'group', $group);
$prev_url = Util::addParameter($refresh_url, 'end', $prevend);
$next_url = Util::addParameter($refresh_url, 'end', $nextend);
$msg_count = sprintf(_("Articles: %d New, %d Old"), $a_new, $a_old);

require TROLL_TEMPLATES . '/common-header.inc';
require TROLL_TEMPLATES . '/menu.inc';
require TROLL_TEMPLATES . '/groupview/header.inc';
require TROLL_TEMPLATES . '/groupview/navbar.inc';
require TROLL_TEMPLATES . '/groupview/article-header.inc';

if ($articles && is_array($overview)) {
    foreach ($overview as $key => $value) {
        if ($value->deleted == 1) {
            continue;
        }
        $messagenum = $value->uid;
        $messageid = $value->message_id;

        $numtokey[$messagenum] = $key;
        $idtonum[$messageid] = $messagenum;
        $numtoid[$messagenum] = $messageid;
        $haskids[$messagenum] = 0;

        $value->references = isset($value->references) ? $value->references : '';
        $parent = chop(strrchr($value->references, "<"));

        if (strlen($parent) > 0 && $prefs->getValue('use_threads')) {
            $x = isset($idtonum[$parent]) ? intval($idtonum[$parent]) : 0;
            if ($x == 0) {
                $indentlevel[$messagenum] = 0;
                $kids[$messagenum] = 0;
            } else {
                $article_parent[$messagenum] = $x;
                $article_kids[$x][] = $messagenum;
                $indentlevel[$messagenum] = $indentlevel[$x]+1;
            }

            if (($messagenum > $last_read) && $prefs->getValue('expand_tree')) {
                $root = $messagenum;
                while ($indentlevel[$root] != 0 && $root > 0) {
                    if ($root > 0) {
                        $totalexpand[$root] = 1;
                    }
                    $root = $article_parent[$root];
                }
                if ($root > 0) {
                    $totalexpand[$root] = 1;
                }
            }
        } else {
            $indentlevel[$messagenum] = 0;
            $kids[$messagenum] = 0;
        }
    }
} else {
    $notification->push(_("There are no posts in this topic."), 'horde.warning');
}

unset($articles); // To be re-set in PrintLine.

if ($prefs->getValue('sortdir')) {
    $c = 0;
    for ($i = $last; ($i > $first) && ($c < $maxresults); $i--) {
        if ($indentlevel[$i] == 0) {
            $c++;
        }
    }

    $first = $i;
    for ($i = $first; $i <= $last; $i++) {
        if (empty($indentlevel[$i])) {
            _printThreadOverview(GetArticleTree($i));
        }
    }
} else { // normal sort
    for ($i = $last; $i >= $first; $i--) {
        if (empty($indentlevel[$i]) && (!empty($totalexpand[$i]) || empty($haskids[$i]))) {
            _printThreadOverview(GetArticleTree($i));
        }
    }

    for ($i = $last; $i >= $first; $i--) {
        if (empty($indentlevel[$i]) && (empty($totalexpand[$i]) && !empty($haskids[$i]))) {
            _printThreadOverview(GetArticleTree($i));
        }
    }
}

if (isset($articles)) {
    $_SESSION['sorted_articles'] = serialize($articles);
    $_SESSION['articles_anchor'] = serialize($art_anchor);
}

$firstprinted--;

$notification->notify(array('listeners' => 'status'));

require TROLL_TEMPLATES . '/groupview/article-footer.inc';
require TROLL_TEMPLATES . '/groupview/navbar.inc';
require TROLL_TEMPLATES . '/groupview/footer.inc';
require $registry->get('templates', 'horde') . '/common-footer.inc';
