<?php

require_once 'Horde/IMAP/Tree.php';

/**
 * The Troll_tree class provides a tree view of the folders in an
 * NNTP repository.  It provides access functions to iterate
 * through this tree and query information about individual
 * mailboxes.
 *
 * $Horde: troll/lib/IMAP/Tree.php,v 1.8 2005/01/03 14:35:36 jan Exp $
 *
 * Copyright 2000-2005 Charles J. Hagenbuch <chuck@horde.org>
 * Copyright 2000-2005 Jon Parise <jon@horde.org>
 * Copyright 2000-2005 Anil Madhavapeddy <avsm@horde.org>
 * Copyright 2003-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @author  Jon Parise <jon@horde.org>
 * @author  Anil Madhavapeddy <avsm@horde.org>
 * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
 * @version $Revision: 1.8 $
 * @since   Troll 0.1
 * @package Troll
 */

class Troll_Tree extends IMAP_Tree {

    /**
     * Singleton method.
     *
     * @see IMAP_Tree::singleton()
     *
     * @access public
     */
    function &singleton($create = false)
    {
        return IMAP_Tree::singleton('troll', 'Troll_Tree', $create, IMAPTREE_INIT_SUB);
    }

    /**
     * Constructor.
     *
     * @see IMAP_Tree::IMAP_Tree()
     */
    function Troll_Tree($init = null, $cachename = null)
    {
        $this->_app = 'troll';
        $this->_delimiter = '.';
        $this->_mode = IMAPTREE_MODE_NEWS;
        $this->_server = Troll::serverString();

        parent::IMAP_Tree(IMAPTREE_INIT_SUB, $cachename);
    }

    /**
     * Subclass specific initialization tasks.
     *
     * @see IMAP_Tree::_init()
     */
    function _init()
    {
        return $this->_getList('%');
    }

    /**
     * Returns a reference to a currently open IMAP stream.
     *
     * @see IMAP_Tree::_getStream()
     */
    function &_getStream()
    {
        return $_SESSION['troll']['stream'];
    }

    /**
     * Initialize/get the list of mailboxes to poll.
     *
     * @see IMAP_Tree::getPollList()
     */
    function getPollList()
    {
        if (is_null($this->_poll)) {
            $this->_poll = array();

            /* Add the user's list of polled mailboxes, if not locked. */
            if (!$GLOBALS['prefs']->isLocked('nav_poll')) {
                $navPollList = @unserialize($GLOBALS['prefs']->getValue('nav_poll'));
                if ($navPollList) {
                    $this->_poll += $navPollList;
                }
            }
        }

        return $this->_poll;
    }

   /**
     * Add element to the poll list.
     *
     * @see IMAP_Tree::addPollList()
     */
    function addPollList($id)
    {
        if (!$GLOBALS['prefs']->isLocked('nav_poll')) {
            $this->getPollList();
            $this->_poll[$id] = true;
            $GLOBALS['prefs']->setValue('nav_poll', serialize($this->_poll));
            $this->_setPolled($this->_tree[$id], true);
      } 
    }

    /**
     * Remove element from the poll list.
     *
     * @see IMAP_Tree::removePollList()
     */
    function removePollList($id)
    {
        if (!$GLOBALS['prefs']->isLocked('nav_poll')) {
            $this->getPollList();
            unset($this->_poll[$id]);
            $GLOBALS['prefs']->setValue('nav_poll', serialize($this->_poll));
            if (isset($this->_tree[$id])) {
                $this->_setPolled($this->_tree[$id], false);
            }
        }
    }

    /**
     * Initalize the list of subscribed newsgroups.
     *
     * @see IMAP_Tree::_initSubscribed()
     */
    function _initSubscribed()
    {
        if (is_null($this->_subscribed)) {
            $s = $GLOBALS['prefs']->getValue('subscriptions');
            $this->_subscribed = (empty($s)) ? array() : @unserialize($s);
        }
    }

    /**
     * Subscribe to newsgroups.
     *
     * @access public
     *
     * @param array $groups  The list of groups to subscribe to.
     */
    function subscribeNewsgroups($groups)
    {
        if (!empty($groups)){
            $this->_initSubscribed();
            $this->_subscribed = array_merge(array_values($groups), $this->_subscribed);
            $GLOBALS['prefs']->setValue('subscriptions', serialize($this->_subscribed));
            foreach ($groups as $val) {
                $this->subscribe($val);
            }
        }
    }

    /**
     * Unsubscribe from newsgroups.
     *
     * @access public
     *
     * @param array $groups  The list of groups to unsubscribe from.
     */
    function unsubscribeNewsgroups($groups)
    {
        if (!empty($groups)){
            $this->_initSubscribed();
            $this->_subscribed = array_diff($this->_subscribed, $groups);
            $GLOBALS['prefs']->setValue('subscriptions', serialize($this->_subscribed));

            /* Remove read message status if there was any. */
            foreach ($groups as $val){
                $this->unsubscribe($val);
                $GLOBALS['troll_driver']->store($val);
            }
        }
    }

}
