<?php
/**
 * $Horde: troll/post.php,v 1.49 2005/01/03 14:35:35 jan Exp $
 *
 * Copyright 2000-2005 Charles J. Hagenbuch <chuck@horde.org>
 * Copyright 2000-2001 Jon Parise <jon@horde.org>
 * Copyright 2000 Anil Madhavapeddy <avsm@horde.org>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

@define('TROLL_BASE', dirname(__FILE__));
require_once TROLL_BASE . '/lib/base.php';
require_once TROLL_BASE . '/lib/version.php';
require_once TROLL_BASE . '/lib/MIME/Headers.php';
require_once TROLL_BASE . '/lib/NNTP.php';
require_once 'Horde/Identity.php';
require_once 'Horde/MIME/Message.php';

$post_error = false;

$group = Util::getFormData('group');
$supersedes = Util::getFormData('supersedes');
$identity_used = Util::getFormData('identity');
$article = Util::getFormData('article');
$message = Util::getFormData('message');
$orig_group = Util::getFormData('orig_group');
$subject = Util::getFormData('subject');
$superseded = Util::getFormData('superseded');
$organization = Util::getFormData('organization');
$references = Util::getFormData('references');

$title = ($group ? $group . ' :: ' : '') . ($supersedes ? _("Supersedes Article") : _("Post Article"));
require TROLL_TEMPLATES . '/common-header.inc';
require TROLL_TEMPLATES . '/post/javascript.inc';

/* Make sure we have an NNTP stream. */
$troll_nntp = &Troll_NNTP::singleton();
$troll_nntp->changeGroup($group);
$nntpob = &$troll_nntp->getNetNNTPOb();

/* Create the identity object. */
$identity = &Identity::singleton(array('troll','troll'));

/* Generate the e-mail select list. */
$select_list = $identity->getSelectList();

if (is_null($message)) {
    $oldsubject = '';
    if (!empty($article)) {
        $art = GetArticle($_SESSION['troll']['stream'], $article);

        $oldsubject = $art['subject'];
        if (!$supersedes &&
            (String::lower(substr($oldsubject, 0, 4)) != 're: ')) {
            $oldsubject = 'Re: ' . $oldsubject;
        }
    }
    require TROLL_TEMPLATES . '/post/form.inc';
} else {
    $from_addr = $identity->getValue('full_name', $identity_used) . ' <' . $identity->getValue('from_addr', $identity_used) . '>';
    $reply_to = $identity->getValue('replyto_addr', Util::getFormData('identity'));

    /* Get the list of unique newsgroups. */
    $newsgroups = array_unique(explode(',', $group));
    $group = implode(',', $newsgroups);

    /* Create the message. */
    $mime = &new MIME_Message();
    $mime->setType('text/plain');

    $ng = count($newsgroups);
    if (!$ng || empty($newsgroups[0])) {
        $notification->push(_("You must select at least 1 newsgroup."), 'horde.error');
        $post_error = true;
    }

    $crosspost = $prefs->getValue('max_crosspost');
    if ($ng > $crosspost) {
        $notification->push(sprintf(_("You are not allowed to crosspost to more than %d groups."), $crosspost), 'horde.error');
        $post_error = true;
    }

    foreach ($newsgroups as $grp) {
        if (is_a($nntpob->selectGroup($grp, 'PEAR_Error'))) {
            $notification->push(_("Cannot set current group") . ' ' . $grp, 'horde.error');
            $post_error = true;
        }
    }

    $fugroups = array_unique(explode(',', Util::getFormData('fu2')));
    $fu = implode(',', $fugroups);
    $fu_count = count($fugroups);

    if (($ng > 1) && ($fu_count == 0)) {
        if ($prefs->getValue('must_fu2')) {
            $notification->push(_("You must set a Followup-To when crossposting."), 'horde.error');
            $post_error = true;
        } else {
            $notification->push(_("You should set a Followup-To when crossposting."), 'horde.warning');
        }
    }

    if ($fu_count) {
        $max_fu = $prefs->getValue('max_fu2');
        if ($fu_count > $max_fu) {
            $notification->push(sprintf(_("You are not allowed to set a follow-up to more than %d groups."), $max_fu), 'horde.error');
            $post_error = true;
        }

        if (!empty($fugroups[0])) {
            foreach ($fu_groups as $grp) {
                if (is_a($nntpob->selectGroup($grp, 'PEAR_Error'))) {
                    if ($grp != 'poster') {
                        $notification->push(_("Cannot set current group") . ' ' . $grp, 'horde.error');
                        $post_error = true;
                    } else {
                        if ($i > 1) {
                            $notification->push(_("'poster' cannot be selected with any group."), 'horde.warning');
                            $fu = 'poster';
                        }
                    }
                }
            }
        }
    }

    if (empty($from)) {
        $notification->push(_("No name (From:)!"), 'horde.error');
        $post_error = true;
    }

    if (Util::getFormData('email') == null) {
        $notification->push(_("No email!"), 'horde.error');
        $post_error = true;
    }

    if (empty($message)) {
        $notification->push(_("No message!"), 'horde.error');
        $post_error = true;
    }

    if (empty($subject)) {
        $notification->push(_("No subject!"), 'horde.error');
        $post_error = true;
    }

    if (isset($article) && (int)$article > 0 && !$post_error) {
        $nntpob->selectGroup($group);
        $old_headers = $nntpob->splitHeaders($article);
        $references = isset($old_headers['References']) ? $old_headers['References'] : '';
        $oldmsgid = $old_headers['Message-ID'];
        if ($supersedes == 'n') {
            $references .= ' ' . $oldmsgid;
        } else {
            $superseded = $oldmsgid;
        }
    }

    if (!$post_error) {
        $from .= ' <' . Util::getFormData('email') . '>';
    }

    /* Add signature. */
    $signature = $identity->getValue('signature', Util::getFormData('identity'));
    if (strlen($signature) > 0) {
        $message .= "\r\n-- \r\n" . $signature;
    }

    /* Create the headers. */
    $headers = &new Troll_Headers();
    $headers->addMessageIdHeader();
    $headers->addHeader('Date', date('r'));
    $headers->addHeader('From', $from_addr);
    $headers->addHeader('Reply-To', ($reply_to) ? $reply_to : $from_addr);
    $headers->addHeader('Sender', 'gateway@' . $conf['server']['name']);
    $headers->addHeader('Subject', $subject);
    $headers->addHeader('Supersedes', $superseded);
    $headers->addHeader('Organization', (!empty($organization)) ? $organization : 'NNRP Gateway at ' . $conf['server']['name']);
    $headers->addHeader('Followup-To', $follow_up);
    $headers->addHeader('References', $references);

    $headers->addMIMEHeaders($mime);

    if (!$post_error) {
        $notification->push(sprintf(_("Sorry, could not post message to %s"), $group), 'horde.error');
        $post_error = true;
    } else {
        $ret = $nntpob->post($subject, $group, $from, $message, $headers->toString());
        if (is_a($ret, 'PEAR_Error')) {
            $notification->push(sprintf(_("Sorry, could not post message to %s"), $group), 'horde.error');
            $post_error = true;
        } else {
            $notification->push(sprintf(_("Message posted to %s"), $group), 'horde.success');
            $nntpob->selectGroup($group);
            $last = $nntpob->last();
            require TROLL_TEMPLATES . '/post/success.inc';
        }
    }
}

echo '<br />';
$notification->notify(array('listeners' => 'status'));

// Second chance
if ($post_error) {
    require TROLL_TEMPLATES . '/post/form.inc';
}

$notFrame = true;
require $registry->get('templates', 'horde') . '/common-footer.inc';
